@@ -5,13 +5,30 @@
 #'      or (ii) a character string in the form \code{xmin,ymin,xmax,ymax}. These
 #'      will be passed to \link{getbb} to be converted to a numerical bounding
 #'      box. Can also be (iii) a matrix representing a bounding polygon as
-#'      returned from `getbb(..., format_out = "polygon")`.
-#' @param nodes_only If `TRUE`, query OSM nodes only. Some OSM structures such
-#'      as `place = "city"` or `highway = "traffic_signals"` are represented by
-#'      nodes only. Queries are built by default to return all nodes, ways, and
-#'      relation, but this can be very inefficient for node-only queries.
-#'      Setting this value to `TRUE` for such cases makes queries more
-#'      efficient, with data returned in the `osm_points` list item.
+#'      returned from `getbb(..., format_out = "polygon")`. To search in an
+#'      area, (iv) a character string with a relation or a (closed) way id in
+#'      the format `"way(id:1)"`, `"relation(id:1, 2)"` or `"relation(id:1, 2,
+#'      3); way(id:2)"` as returned by `getbb(..., format_out = "osm_type_id")`
+#'      or \link{bbox_to_string} with a `data.frame` from `getbb(..., format_out
+#'      = "data.frame")` to select all areas combined (relations and ways).
+#' @param nodes_only WARNING: this parameter is equivalent to
+#'      `osm_types = "node"` and will be DEPRECATED. If `TRUE`, query OSM nodes
+#'      only. Some OSM structures such as `place = "city"` or
+#'      `highway = "traffic_signals"` are represented by nodes only. Queries are
+#'      built by default to return all nodes, ways, and relation, but this can
+#'      be very inefficient for node-only queries. Setting this value to `TRUE`
+#'      for such cases makes queries more efficient, with data returned in the
+#'      `osm_points` list item.
+#' @param osm_types A character vector with several OSM types to query: `node`,
+#'      `way` and `relation` is the default. `nwr`, `nw`, `wr`, `nr` and `rel`
+#'      are also valid types. Ignored if `nodes_only = TRUE`.
+#'      `osm_types = "node"` is equivalent to `nodes_only = TRUE`.
+#' @param out The level of verbosity of the overpass result: `body` (geometries
+#'      and tags, the default), `tags` (tags without geometry), `meta` (like
+#'      body + Timestamp, Version, Changeset, User, User ID of the last
+#'      edition), `skel` (geometries only), `tags center` (tags without geometry
+#'      + the coordinates of the center of the bounding box) and `ids` (type and
+#'      id of the objects only).
 #' @param datetime If specified, a date and time to extract data from the OSM
 #'      database as it was up to the specified date and time, as described at
 #'      \url{https://wiki.openstreetmap.org/wiki/Overpass_API/Overpass_QL#date}.
@@ -20,7 +37,11 @@
 #' @param datetime2 If specified, return the \emph{difference} in the OSM
 #'      database between \code{datetime} and \code{datetime2}, where
 #'      \code{datetime2 > datetime}. See
-#'      \url{https://wiki.openstreetmap.org/wiki/Overpass_API/Overpass_QL#Delta_between_two_dates_.28.22diff.22.29}.
+#'      \url{https://wiki.openstreetmap.org/wiki/Overpass_API/Overpass_QL#Difference_between_two_dates_(diff)}.
+#' @param adiff If `TRUE`, query for [augmented
+#'      difference](https://wiki.openstreetmap.org/wiki/Overpass_API/Overpass_QL#Augmented-difference_between_two_dates_(adiff)).
+#'      The result indicates what happened to the modified and deleted OSM
+#'      objects. Requires `datetime(2)*`.
 #' @param timeout It may be necessary to increase this value for large queries,
 #'      because the server may time out before all data are delivered.
 #' @param memsize The default memory size for the 'overpass' server in *bytes*;
@@ -28,8 +49,17 @@
 #'
 #' @return An `overpass_query` object
 #'
+#' @details The `out` statement for `tags`, `tags center`and `id`, do not return
+#' geometries. Neither `out = "meta"` nor `adiff = TRUE` options are implemented
+#' for all `osmdata_*` functions yet. Use [osmdata_xml] or [osmdata_data_frame]
+#' to get the result of these queries. See the documentation of the [out
+#' statement](https://wiki.openstreetmap.org/wiki/Overpass_API/Overpass_QL#out)
+#' and [augmented
+#' difference](https://wiki.openstreetmap.org/wiki/Overpass_API/Overpass_QL#Augmented-difference_between_two_dates_(adiff))
+#' for more details about these options.
+#'
 #' @note See
-#' /url{https://wiki.openstreetmap.org/wiki/Overpass_API#Resource_management_options_.28osm-script.29}
+#' \url{https://wiki.openstreetmap.org/wiki/Overpass_API#Resource_management_options_.28osm-script.29}
 #' for explanation of `timeout` and `memsize` (or `maxsize` in overpass terms).
 #' Note in particular the comment that queries with arbitrarily large `memsize`
 #' are likely to be rejected.
@@ -57,18 +87,63 @@
 #' opq <- opq (bbox, nodes_only = TRUE) %>%
 #'     add_osm_feature (key = "place", value = "city") %>%
 #'     osmdata_sf (quiet = FALSE)
+#'
+#' # Filter by a search area
+#' qa1 <- getbb ("Catalan Countries", format_out = "osm_type_id") %>%
+#'     opq (nodes_only = TRUE) %>%
+#'     add_osm_feature (key = "capital", value = "4")
+#' opqa1 <- osmdata_sf (qa1)
+#' # Filter by a multiple search areas
+#' bb <- getbb ("Vilafranca", format_out = "data.frame")
+#' qa2 <- bbox_to_string (bb [bb$osm_type != "node", ]) %>%
+#'     opq (nodes_only = TRUE) %>%
+#'     add_osm_feature (key = "place")
+#' opqa2 <- osmdata_sf (qa2)
 #' }
 opq <- function (bbox = NULL, nodes_only = FALSE,
-                 datetime = NULL, datetime2 = NULL,
+                 osm_types = c ("node", "way", "relation"),
+                 out = c ("body", "tags", "meta", "skel", "tags center", "ids"),
+                 datetime = NULL, datetime2 = NULL, adiff = FALSE,
                  timeout = 25, memsize) {
 
     timeout <- format (timeout, scientific = FALSE)
     prefix <- paste0 ("[out:xml][timeout:", timeout, "]")
-    suffix <- ifelse (
-        nodes_only,
-        "); out;",
-        ");\n(._;>;);\nout body;"
-    ) # recurse down
+
+    if (nodes_only) {
+        osm_types <- "node"
+    } else {
+        osm_types <- try (
+            match.arg (osm_types,
+                choices = c (
+                    "node", "way", "rel", "relation",
+                    "nwr", "nw", "wr", "nr"
+                ),
+                several.ok = TRUE
+        ), silent = TRUE)
+        if (inherits (osm_types, "try-error")) {
+            stop ('osm_types parameter must be a vector with values from ',
+                '"node", "way", "rel", "relation", ',
+                '"nwr", "nw", "wr" and "nr".',
+                call. = FALSE
+            )
+        }
+    }
+
+    out <- try (match.arg (out), silent = TRUE)
+    if (inherits (out, "try-error")) {
+        stop ("out parameter must be ",
+            '"body", "tags", "meta", "skel", "tags center" or "ids".',
+            call. = FALSE
+        )
+    }
+
+    has_geometry <- !nodes_only && out %in% c ("body", "meta", "skel")
+    if (has_geometry) {
+        suffix <- paste0 (");\n(._;>;);\nout ", out, ";") # recurse down
+    } else {
+        suffix <- paste0 ("); out ", out, ";")
+    }
+
     if (!missing (memsize)) {
         prefix <- paste0 (
             prefix, "[maxsize:", # nocov
@@ -77,11 +152,22 @@ opq <- function (bbox = NULL, nodes_only = FALSE,
     } # nocov
     if (!is.null (datetime)) {
 
-        datetime <- check_datetime (datetime)
+        if (!is_datetime (datetime)) {
+            stop (
+                "datetime must be in ISO8601 format ('YYYY-MM-DDThh:mm:ssZ')",
+                call. = FALSE
+            )
+        }
+
 
         if (!is.null (datetime2)) {
 
-            datetime2 <- check_datetime (datetime2)
+            if (!is_datetime (datetime2)) {
+                stop (
+                    "datetime2 must be in ISO8601 format ('YYYY-MM-DDThh:mm:ssZ')",
+                    call. = FALSE
+                )
+            }
             prefix <- paste0 (
                 '[diff:\"', datetime, '\",\"', datetime2, '\"]',
                 prefix
@@ -90,13 +176,20 @@ opq <- function (bbox = NULL, nodes_only = FALSE,
 
             prefix <- paste0 ('[date:\"', datetime, '\"]', prefix)
         }
+
+        if (adiff) {
+            prefix <- gsub ("^\\[(diff|date):", "[adiff:", prefix)
+        }
     }
 
     res <- list (
         bbox = bbox_to_string (bbox),
         prefix = paste0 (prefix, ";\n(\n"),
-        suffix = suffix, features = NULL
+        suffix = suffix,
+        features = NULL,
+        osm_types = osm_types
     )
+
     class (res) <- c (class (res), "overpass_query")
     attr (res, "datetime") <- datetime
     attr (res, "datetime2") <- datetime2
@@ -105,46 +198,22 @@ opq <- function (bbox = NULL, nodes_only = FALSE,
     return (res)
 }
 
-check_datetime <- function (x) {
-
-    if (nchar (x) != 20 &
-        substring (x, 5, 5) != "-" &
-        substring (x, 8, 8) != "-" &
-        substring (x, 11, 11) != "T" &
-        substring (x, 14, 14) != ":" &
-        substring (x, 17, 17) != ":" &
-        substring (x, 20, 20) != "Z") {
-        stop ("datetime must be in ISO8601 format ('YYYY-MM-DDThh:mm:ssZ')")
-    }
-    YY <- substring (x, 1, 4) # nolint
-    MM <- substring (x, 6, 7) # nolint
-    DD <- substring (x, 9, 10) # nolint
-    hh <- substring (x, 12, 13)
-    mm <- substring (x, 15, 16)
-    ss <- substring (x, 18, 19)
-    if (formatC (as.integer (YY), width = 4, flag = "0") != YY |
-        formatC (as.integer (MM), width = 2, flag = "0") != MM |
-        formatC (as.integer (DD), width = 2, flag = "0") != DD |
-        formatC (as.integer (hh), width = 2, flag = "0") != hh |
-        formatC (as.integer (mm), width = 2, flag = "0") != mm |
-        formatC (as.integer (ss), width = 2, flag = "0") != ss) {
-        stop ("x is not is ISO8601 format ('YYYY-MM-DDThh:mm:ssZ')")
-    }
-    invisible (x)
-}
-
 # used in the following add_osm_feature fn
 paste_features <- function (key, value, key_pre = "", bind = "=",
                             match_case = FALSE, value_exact = FALSE) {
     if (is.null (value)) {
 
-        feature <- paste0 (sprintf (' ["%s"]', key))
+        feature <- ifelse (substring (key, 1, 1) == "!",
+                sprintf ('[!"%s"]', substring (key, 2, nchar (key))),
+                sprintf ('["%s"]', key)
+        )
+
     } else {
 
         if (length (value) > 1) {
 
             # convert to OR'ed regex:
-            value <- paste0 (value, collapse = "|")
+            value <- paste (value, collapse = "|")
             if (value_exact) {
                 value <- paste0 ("^(", value, ")$")
             }
@@ -160,7 +229,7 @@ paste_features <- function (key, value, key_pre = "", bind = "=",
             }
         }
         feature <- paste0 (sprintf (
-            ' [%s"%s"%s"%s"',
+            '[%s"%s"%s"%s"',
             key_pre, key, bind, value
         ))
         if (!match_case) {
@@ -175,7 +244,8 @@ paste_features <- function (key, value, key_pre = "", bind = "=",
 #' Add a feature to an Overpass query
 #'
 #' @param opq An `overpass_query` object
-#' @param key feature key
+#' @param key feature key; can be negated with an initial exclamation mark,
+#' `key = "!this"`, and can also be a vector if `value` is missing.
 #' @param value value for feature key; can be negated with an initial
 #' exclamation mark, `value = "!this"`, and can also be a vector,
 #' `value = c ("this", "that")`.
@@ -233,6 +303,13 @@ paste_features <- function (key, value, key_pre = "", bind = "=",
 #' # Use of negation to extract all non-primary highways
 #' q <- opq ("portsmouth uk") %>%
 #'     add_osm_feature (key = "highway", value = "!primary")
+#'
+#' # key negation without warnings
+#' q3 <- opq ("Vinçà", osm_type="node") %>%
+#'     add_osm_feature (key = c("name", "!name:ca"))
+#' q4 <- opq ("el Carxe", osm_type="node") %>%
+#'     add_osm_feature (key = "natural", value = "peak") %>%
+#'     add_osm_feature (key = "!ele")
 #' }
 add_osm_feature <- function (opq,
                              key,
@@ -241,50 +318,46 @@ add_osm_feature <- function (opq,
                              value_exact = TRUE,
                              match_case = TRUE,
                              bbox = NULL) {
-
     if (missing (key)) {
         stop ("key must be provided")
     }
 
-    if (is.null (bbox) & is.null (opq$bbox)) {
+    if (is.null (bbox) && is.null (opq$bbox)) {
         stop ("Bounding box has to either be set in opq or must be set here")
     }
 
-    if (is.null (bbox)) {
-        bbox <- opq$bbox
-    } else {
+    if (!is.null (bbox)) {
         bbox <- bbox_to_string (bbox)
         opq$bbox <- bbox
     }
 
-    if (!key_exact & value_exact) {
-        message (
-            "key_exact = FALSE can only combined with ",
-            "value_exact = FALSE; setting value_exact = FALSE"
-        )
-        value_exact <- FALSE
-    }
-
-    if (value_exact) {
-        bind <- "="
-    } else {
-        bind <- "~"
-    }
-    key_pre <- ""
-    if (!key_exact) {
-        key_pre <- "~"
-    }
+    bind_key_pre <- set_bind_key_pre (key_exact, value_exact)
 
     if (missing (value)) {
         value <- NULL
     }
 
     feature <- paste_features (
-        key, value, key_pre, bind,
+        key, value, bind_key_pre$key_pre, bind_key_pre$bind,
         match_case, value_exact
     )
+    feature<- paste (feature, collapse = " ")
 
-    opq$features <- paste0 (c (opq$features, feature), collapse = "")
+    if (is.null (opq$features)) {
+        opq$features <- feature
+    } else {
+        opq$features <- paste (opq$features, feature)
+    }
+
+    if (any (w <- !grepl("\\[(\\\"|~)", opq$features))) {
+        warning(
+            "The query will request objects whith only a negated key (",
+            paste (opq$features[w], collapse = ", "), ") , which can be quite ",
+            "expensive for overpass servers. Add other features or be shure ",
+            "that that is what you want. To avoid this warning, reorder your ",
+            "calls to add_osm_feature/s and leave key negations at the end."
+        )
+    }
 
     if (is.null (opq$suffix)) {
         opq$suffix <- ");\n(._;>;);\nout body;"
@@ -296,19 +369,112 @@ add_osm_feature <- function (opq,
     opq
 }
 
+#' Get conditional operator/prefix values based on value_exact and key_exact
+#'
+#' @param bind Operator used to combine key and value. Options include "="
+#'   (default - equivalent to `value_exact = TRUE`), "!=", "~" (equivalent to
+#'   `value_exact = FALSE`), or "!~".
+#' @param key_pre Prefix for key. Options include "" or "~".
+#' @noRd
+set_bind_key_pre <- function (key_exact = TRUE,
+                              value_exact = TRUE,
+                              features = NULL,
+                              bind = "=",
+                              key_pre = "") {
+    if (!is.null (value_exact)) {
+        value_exact <- check_value_exact (value_exact, key_exact)
+    }
+
+    check_bind_key_pre (bind, key_pre)
+
+    if (!is.null (features)) {
+        if (length (bind) == 1) {
+            bind <- rep_len (bind, length (features))
+        } else if (!identical_length (features, bind)) {
+            stop (
+                "bind must be length 1 or the same length as features"
+            )
+        }
+
+        if (length (key_pre) == 1) {
+            key_pre <- rep_len (key_pre, length (features))
+        } else if (!identical_length (features, key_pre)) {
+            stop (
+                "key_pre must be length 1 or the same length as features"
+            )
+        }
+    }
+
+    features_len <- 1L
+    if (!is.null (features)) {
+        features_len <- length (features)
+    }
+
+    if (!is.null (value_exact) && !value_exact) {
+        bind <- rep_len ("~", features_len)
+    }
+
+    if (!is.null (key_exact) && !key_exact) {
+        key_pre <- rep_len ("~", features_len)
+    }
+
+    list (
+        "bind" = bind,
+        "key_pre" = key_pre
+    )
+}
+
+#' Are x and y an identical length?
+#'
+#' @noRd
+identical_length <- function (x, y) {
+    identical (length (x), length (y))
+}
+
+#' Check that value_exact can be combined with key_exact
+#'
+#' @noRd
+check_value_exact <- function (value_exact = TRUE, key_exact = TRUE) {
+    if (value_exact && !key_exact) {
+        message (
+            "key_exact = FALSE can only combined with ",
+            "value_exact = FALSE; setting value_exact = FALSE"
+        )
+
+        return (FALSE)
+    }
+
+    value_exact
+}
+
+#' Check for valid bind and key_pre values
+#'
+#' @noRd
+check_bind_key_pre <- function (bind = "=", key_pre = "") {
+    if (!all (bind %in% c ("=", "!=", "~", "!~"))) {
+        stop ('bind must only include "=", "!=", "~", or "!~"')
+    }
+
+    if (!all (key_pre %in% c ("", "~"))) {
+        stop ('key_pre must only include "" or "~"')
+    }
+}
+
 #' Add multiple features to an Overpass query
 #'
 #' Alternative version of \link{add_osm_feature} for creating single queries
-#' with multiple features. Key-value matching may be controlled by using
-#' the filter symbols described in
+#' with multiple features. Key-value matching may be controlled by using the
+#' filter symbols described in
 #' \url{https://wiki.openstreetmap.org/wiki/Overpass_API/Overpass_QL#By_tag_.28has-kv.29}.
 #'
 #' @inheritParams add_osm_feature
 #' @inheritSection add_osm_feature `add_osm_feature` vs `add_osm_features`
-#' @param features Character vector of key-value pairs with keys and values
-#' enclosed in escape-formatted quotations (see examples).
+#' @param features A named list or vector with the format `list("<key>" =
+#'      "<value>")` or `c("<key>" = "<value>")` or a character vector of
+#'      key-value pairs with keys and values enclosed in escape-formatted
+#'      quotations. See examples for details.
 #' @param bbox optional bounding box for the feature query; must be set if no
-#'        opq query bbox has been set
+#'      opq query bbox has been set.
 #' @return \link{opq} object
 #'
 #' @references \url{https://wiki.openstreetmap.org/wiki/Map_Features}
@@ -320,6 +486,12 @@ add_osm_feature <- function (opq,
 #' @examples
 #' \dontrun{
 #' q <- opq ("portsmouth usa") %>%
+#'     add_osm_features (features = list (
+#'         "amenity" = "restaurant",
+#'         "amenity" = "pub"
+#'     ))
+#'
+#' q <- opq ("portsmouth usa") %>%
 #'     add_osm_features (features = c (
 #'         "\"amenity\"=\"restaurant\"",
 #'         "\"amenity\"=\"pub\""
@@ -336,19 +508,14 @@ add_osm_feature <- function (opq,
 #' }
 add_osm_features <- function (opq,
                               features,
-                              bbox = NULL) {
-
-    if (missing (features)) {
-        stop ("features must be provided")
-    }
-
-    if (is.null (bbox) & is.null (opq$bbox)) {
+                              bbox = NULL,
+                              key_exact = TRUE,
+                              value_exact = TRUE) {
+    if (is.null (bbox) && is.null (opq$bbox)) {
         stop ("Bounding box has to either be set in opq or must be set here")
     }
 
-    if (is.null (bbox)) {
-        bbox <- opq$bbox
-    } else {
+    if (!is.null (bbox)) {
         bbox <- bbox_to_string (bbox)
         opq$bbox <- bbox
     }
@@ -357,11 +524,26 @@ add_osm_features <- function (opq,
         opq$suffix <- ");\n(._;>;);\nout body;"
     }
 
-    if (length (which (!grepl ("\\\"", features))) > 0L) {
-        stop (
-            "features must be enclosed in escape-delimited ",
-            "quotations (see example)"
+    check_features (features)
+
+    if (is_named (features)) {
+        bind_key_pre <-
+            set_bind_key_pre (
+                features = features,
+                value_exact = value_exact,
+                key_exact = key_exact
+            )
+
+        features <- mapply (function (key, value, key_pre, bind) {
+                paste_features (key, value, key_pre = key_pre, bind = bind,
+                    match_case = TRUE, value_exact = value_exact)
+            },
+            key = names (features), value = features,
+            key_pre = bind_key_pre$key_pre, bind = bind_key_pre$bind,
+            SIMPLIFY = FALSE
         )
+        features <- as.character (features)
+
     }
 
     index <- which (!grepl ("^\\[", features))
@@ -369,19 +551,58 @@ add_osm_features <- function (opq,
     index <- which (!grepl ("\\]$", features))
     features [index] <- paste0 (features [index], "]")
 
-    opq$features <- features
+    opq$features <- unique (c (opq$features, features))
 
     opq
 }
 
+#' Is features a named list or character vector?
+#'
+#' @noRd
+is_named <- function (x) {
+    !is.null (names (x)) && !any ("" %in% names (x))
+}
+
+#' Is features an escape-delimited string?
+#'
+#' @noRd
+is_escape_delimited <- function (x) {
+    length (which (!grepl ("\\\"", x))) > 0L
+}
+
+#' Check if features is provided and uses the required class and formatting
+#'
+#' @noRd
+check_features <- function (features) {
+    if (missing (features)) {
+        stop ("features must be provided", call. = FALSE)
+    }
+
+    stopifnot (
+        "features must be a list or character vector." =
+            is.character (features) | is.list (features)
+    )
+
+    if (!is_named (features) && is_escape_delimited (features)) {
+        stop (
+            "features must be a named list or vector or a character vector ",
+            "enclosed in escape delimited quotations (see examples)",
+            call. = FALSE
+        )
+    }
+}
+
 #' Add a feature specified by OSM ID to an Overpass query
 #'
 #' @param id One or more official OSM identifiers (long-form integers), which
-#' must be entered as either a character or *numeric* value (because R does not
-#' support long-form integers).
-#' @param type Type of object; must be either `node`, `way`, or `relation`
+#'      must be entered as either a character or *numeric* value (because R does
+#'      not support long-form integers). id can also be a character string
+#'      prefixed with the id type, e.g. "relation/11158003"
+#' @param type Type of objects (recycled); must be either `node`, `way`, or
+#'      `relation`. Optional if id is prefixed with the type.
 #' @param open_url If `TRUE`, open the OSM page of the specified object in web
-#' browser. Multiple objects (`id` values) will be opened in multiple pages.
+#'      browser. Multiple objects (`id` values) will be opened in multiple
+#'      pages.
 #' @return \link{opq} object
 #'
 #' @references
@@ -408,31 +629,50 @@ add_osm_features <- function (opq,
 #'     osmdata_sf ()
 #' dat2$osm_lines # the desired ways
 #' dat <- c (dat1, dat2) # The node and way data combined
+#' # All in one (same result as dat)
+#' id <- c (1489221200, 1489221321, 1489221491, 136190595, 136190596)
+#' type <- c ("node", "node", "node", "way", "way")
+#' datAiO <- opq_osm_id (id = id, type = type) %>%
+#'     opq_string () %>%
+#'     osmdata_sf ()
 #' }
 opq_osm_id <- function (id = NULL, type = NULL, open_url = FALSE) {
-
     if (is.null (type)) {
-        stop ("type must be specified: one of node, way, or relation")
+        if (is.null (id)) {
+            stop (
+                "type must be specified: one of ",
+                "node, way, or relation if id is 'NULL'",
+                call. = FALSE
+            )
+        } else if (all (grepl ("^node/|^way/|^relation/", id))) {
+            type <- dirname (id)
+            id <- basename (id)
+        }
+    }
+
+    type <- tolower (type)
+    if (!all (type %in% c ("node", "way", "relation"))) {
+        stop ('type items must be "node", "way" or "relation".')
     }
-    type <- match.arg (tolower (type), c ("node", "way", "relation"))
 
     if (is.null (id)) {
         stop ("id must be specified.")
     }
-    if (!(is.character (id) | storage.mode (id) == "double")) {
+    if (!(is.character (id) || storage.mode (id) == "double")) {
         stop ("id must be character or numeric.")
     }
-    if (length (id) != 1L) {
-        stop ("Only a single id may be entered.")
+    if (length (id) %% length (type) != 0 || length (type) > length (id)) {
+        stop ("id length must be a multiple of type length.")
     }
 
     if (!is.character (id)) {
-        id <- paste0 (id)
+        id <- as.character (id)
     }
 
     opq <- opq (1:4)
     opq$bbox <- NULL
     opq$features <- NULL
+    opq$osm_types <- NULL
     opq$id <- list (type = type, id = id)
 
     if (open_url) {
@@ -483,11 +723,11 @@ opq_enclosing <- function (lon = NULL, lat = NULL,
 
     enclosing <- match.arg (tolower (enclosing), c ("relation", "way"))
 
-    if (is.null (lon) | is.null (lat)) {
+    if (is.null (lon) || is.null (lat)) {
         stop ("'lon' and 'lat' must be provided.")
     }
-    if (!(is.numeric (lon) & is.numeric (lat) &
-        length (lon) == 1L & length (lat) == 1L)) {
+    if (!(is.numeric (lon) && is.numeric (lat) &&
+        length (lon) == 1L && length (lat) == 1L)) {
         stop ("'lon' and 'lat' must both be single numeric values.")
     }
 
@@ -597,12 +837,18 @@ opq_string_intern <- function (opq, quiet = TRUE) {
 
     lat <- lon <- NULL # suppress no visible binding messages
 
+    if (attr (opq, "nodes_only")) {
+        opq$osm_types <- "node"
+    }
+
+    map_to_area <- grepl ("(node|way|relation|rel)\\(id:[0-9, ]+\\)", opq$bbox)
+
     res <- NULL
     if (!is.null (opq$features)) { # opq with add_osm_feature
 
         features <- opq$features
 
-        if (length (features) > 1L) { # from opq_features fn
+        if (length (features) > 1L) { # from add_osm_features fn
 
             features <- vapply (features, function (i) {
                 paste (i, collapse = "")
@@ -612,15 +858,7 @@ opq_string_intern <- function (opq, quiet = TRUE) {
             )
         }
 
-        if (attr (opq, "nodes_only")) {
-
-            features <- paste0 (sprintf (
-                " node %s (%s);\n",
-                features,
-                opq$bbox
-            ))
-
-        } else if (!is.null (attr (opq, "enclosing"))) {
+        if (!is.null (attr (opq, "enclosing"))) {
 
             if (length (features) > 1) {
                 stop ("enclosing queries can only accept one feature")
@@ -639,35 +877,38 @@ opq_string_intern <- function (opq, quiet = TRUE) {
 
         } else {
 
-            features <- paste0 (
-                sprintf (
-                    " node %s (%s);\n",
-                    features,
-                    opq$bbox
-                ),
-                sprintf (
-                    " way %s (%s);\n",
-                    features,
-                    opq$bbox
-                ),
-                sprintf (
-                    " relation %s (%s);\n\n",
-                    features,
-                    opq$bbox
+            if (!map_to_area) {
+                features <-  c (sprintf ("  %s %s (%s);\n",
+                                         opq$osm_types, features, opq$bbox)
                 )
-            )
+            } else {
+                opq$prefix <- gsub ("\n$", "", opq$prefix)
+                search_area <- paste0 (
+                    opq$bbox,
+                    "; map_to_area->.searchArea; );\n(\n"
+                )
+                features <- c (
+                    search_area,
+                    sprintf ("  %s %s (area.searchArea);\n", opq$osm_types, features)
+                )
+            }
         }
 
         res <- paste0 (
             opq$prefix,
-            paste0 (features, collapse = ""),
+            paste (features, collapse = ""),
             opq$suffix
         )
 
     } else if (!is.null (opq$id)) { # opq with opq_osm_id
 
-        id <- paste (opq$id$id, collapse = ",")
-        id <- sprintf (" %s(id:%s);\n", opq$id$type, id)
+        type_id <- data.frame (type = opq$id$type, id = opq$id$id)
+        type_id <- split (type_id, type_id$type)
+        id <- mapply (function (type, ids) {
+            paste0 (" ", type, "(id:", paste (ids$id, collapse = ","), ");\n")
+        }, type = names (type_id), ids = type_id)
+
+        id <- paste (id, collapse = "")
         res <- paste0 (opq$prefix, id, opq$suffix)
 
     } else { # straight opq with neither features nor ID specified
@@ -681,13 +922,19 @@ opq_string_intern <- function (opq, quiet = TRUE) {
             )
         }
 
-        bbox <- paste0 (
-            sprintf (" node (%s);\n", opq$bbox),
-            sprintf (" way (%s);\n", opq$bbox),
-            sprintf (" relation (%s);\n", opq$bbox)
-        )
+        if (!map_to_area) {
+            bbox <- sprintf ("  %s (%s);\n", opq$osm_types, opq$bbox)
+        } else {
+                opq$prefix <- gsub ("\n$", "", opq$prefix)
+            search_area <-
+                paste0 (opq$bbox, "; map_to_area->.searchArea; );\n(\n")
+            bbox <- c (
+                search_area,
+                sprintf ("  %s (area.searchArea);\n", opq$osm_types)
+            )
+        }
 
-        res <- paste0 (opq$prefix, bbox, opq$suffix)
+        res <- paste0 (opq$prefix, paste (bbox, collapse = ""), opq$suffix)
     }
 
     return (res)
