@@ -126,3 +126,30 @@ test_that ("ways", {
         expect_identical (attr (g, a), attr (g_sf, a))
     }
 })
+
+
+test_that ("non-valid key names", {
+    osm_multi <- test_path ("fixtures", "osm-multi.osm")
+    q0 <- opq (bbox = c (1, 1, 5, 5))
+    x <- osmdata_sf (q0, osm_multi)
+
+    k <- lapply (x[grep ("osm_", names(x))], function (f) {
+        expect_true ("name:ca" %in% names (f))
+    })
+})
+
+test_that ("clashes in key names", {
+    osm_multi_key_clashes <- test_path ("fixtures", "osm-key_clashes.osm")
+    q0 <- opq (bbox = c (1, 1, 5, 5))
+    expect_warning(
+        x <- osmdata_sf (q0, osm_multi_key_clashes),
+        "Feature keys clash with id or metadata columns and will be renamed by "
+    )
+
+    expect_false (any (duplicated (names (x$osm_points))))
+    # x$osm_points don't have osm_id column in tags TODO?
+    k <- lapply (x[grep ("osm_", names (x))[-1]], function (f) {
+        expect_false (any (duplicated (names (f))))
+        expect_true (all (c ("osm_id", "osm_id.1") %in% names (f)))
+    })
+})
