@@ -1,14 +1,10 @@
 context ("unname-osmdata-sf")
 
+skip_if (!test_all)
+
 has_internet <- curl::has_internet ()
 skip_if (!has_internet)
 
-test_all <- (identical (Sys.getenv ("MPADGE_LOCAL"), "true") |
-    identical (Sys.getenv ("GITHUB_WORKFLOW"), "test-coverage"))
-skip_if (!test_all)
-
-set_overpass_url ("https://overpass-api.de/api/interpreter")
-
 require (sf)
 
 test_that ("unname", {
